var cachedSettings = {};

function toggleMode(toggle, id) {
  gantt.$zoomToFit = !gantt.$zoomToFit;
  if (gantt.$zoomToFit) {
    if (Gantt[id] !== undefined) {
      //Saving previous scale state for future restore
      saveConfig(Gantt[id]);
      zoomToFit(Gantt[id], id);
    }
    else {
      //Saving previous scale state for future restore
      saveConfig(gantt);
      zoomToFit(gantt, id);
    }

  }
  else {
    if (Gantt[id] !== undefined) {
      //Restore previous scale state
      restoreConfig(Gantt[id]);
      Gantt[id].render();
    }
    else {
      //Restore previous scale state
      restoreConfig(gantt);
      gantt.render();
    }
  }
}


function saveConfig(gantt) {
  let config = gantt.config;
  cachedSettings = {};
  cachedSettings.scales = config.scales;
  cachedSettings.start_date = config.start_date;
  cachedSettings.end_date = config.end_date;
  cachedSettings.scroll_position = gantt.getScrollState();
}

function restoreConfig(gantt) {
  applyConfig(gantt, cachedSettings);
}

function applyConfig(gantt, config, dates, id = 0) {
  gantt.config.scales = config.scales;

  // restore the previous scroll position
  if (config.scroll_position) {
    setTimeout(function () {
      gantt.scrollTo(config.scroll_position.x, config.scroll_position.y)
    }, 4)
  }
}


function zoomToFit(gantt, id = 0) {
  let project = gantt.getSubtaskDates();
  let areaWidth = gantt.$task.offsetWidth;
  let scaleConfigs = zoomConfig.levels;
  for (var i = 0; i < scaleConfigs.length; i++) {
    let columnCount = getUnitsBetween(project.start_date, project.end_date, scaleConfigs[i].scales[scaleConfigs[i].scales.length - 1].unit, scaleConfigs[i].scales[0].step);
    if ((columnCount + 2) * gantt.config.min_column_width <= areaWidth) {
      break;
    }
  }
  if (i === scaleConfigs.length) {
    i--;
  }

  gantt.ext.zoom.setLevel(scaleConfigs[i].name);
  applyConfig(scaleConfigs[i], project, id);
}

// Get number of columns in timeline.
function getUnitsBetween(from, to, unit, step) {
  var start = new Date(from),
    end = new Date(to);
  var units = 0;
  while (start.valueOf() < end.valueOf()) {
    units++;
    start = gantt.date.add(start, step, unit);
  }
  return units;
}

function zoom_in(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.zoom.zoomOut();
    Gantt[id].$zoomToFit = false;
  }
  else {
    gantt.ext.zoom.zoomOut();
    gantt.$zoomToFit = false;
  }
}

function zoom_out(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.zoom.zoomIn();
    Gantt[id].$zoomToFit = false;
  }
  else {
    gantt.ext.zoom.zoomIn();
    gantt.$zoomToFit = false;
  }
}

function set_scale(e, id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.zoom.setLevel(e.value);
  }
  else {
    gantt.ext.zoom.setLevel(e.value);
  }
}

function collapse(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].batchUpdate(function () {
      Gantt[id].eachTask(function (task) {
        Gantt[id].close(task.id)
      });
    });
  }
  else {
    gantt.batchUpdate(function () {
      gantt.eachTask(function (task) {
        gantt.close(task.id)
      });
    });
  }
}

function expand(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].batchUpdate(function () {
      Gantt[id].eachTask(function (task) {
        Gantt[id].open(task.id)
      });
    });
  }
  else {
    gantt.batchUpdate(function () {
      gantt.eachTask(function (task) {
        gantt.open(task.id)
      });
    });
  }
}

function toggleCollapse(e, id) {
  let action = e.dataset.action;
  let listElement = e.closest(".wrapper-controls").querySelectorAll("[onclick^='toggleCollapse']");
  listElement.forEach(function (element) {
    element.style.display = 'inline-block';
  });
  e.closest(".wrapper-controls").querySelector("[onclick^='toggleCollapse'][data-action='" + action + "']").style.display = 'none';
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].batchUpdate(function () {
      Gantt[id].eachTask(function (task) {
        Gantt[id][action](task.id)
      });
    });
  }
  else {
    gantt.batchUpdate(function () {
      gantt.eachTask(function (task) {
        gantt[action](task.id)
      });
    });
  }
}

function undo(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].undo();
  }
  else {
    gantt.undo();
  }
}

function redo(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].redo();
  }
  else {
    gantt.redo();
  }
}

function exportToExcel(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].exportToExcel();
  }
  else {
    gantt.exportToExcel();
  }
}

function exportToMSProject(id, config) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].exportToMSProject(config);
  }
  else {
    gantt.exportToMSProject(config);
  }
}

function exportToICal(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].exportToICal();
  }
  else {
    gantt.exportToICal();
  }
}

function exportToPDF(id) {
  let config = {
    raw: true,
    name: document.title + '.pdf',
    locale: navigator.language
  }

  jQuery.when(jQuery.get(decodeURIComponent(drupalSettings['link_css'])))
    .always(function(response) {
      if (response.length) {
        if (config.hasOwnProperty('header')) {
          config.header += '<style>' + response + '</style>';
        }else {
          config.header = '<style>' + response + '</style>';
        }
      }

      if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
        Gantt[id].exportToPDF(config);
      }
      else {
        gantt.exportToPDF(config);
      }
    });
}

function exportToPNG(id) {
  let config = {
    raw: true,
    name: document.title + '.png',
    locale: navigator.language
  }

  jQuery.when(jQuery.get(decodeURIComponent(drupalSettings['link_css'])))
    .always(function(response) {
      console.log('1')
      if (response.length) {
        if (config.hasOwnProperty('header')) {
          config.header += '<style>' + response + '</style>';
        }else {
          config.header = '<style>' + response + '</style>';
        }
      }

      if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
        Gantt[id].exportToPNG(config);
      }
      else {
        gantt.exportToPNG(config);
      }
    });
}

function fullscreen(id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].ext.fullscreen.toggle();
  }
  else {
    gantt.ext.fullscreen.toggle();
  }
}

var zoomConfig = {
  levels: [
    // Hours.
    {
      name: "hour",
      scale_height: 27,
      scales: [
        {unit: "day", step: 1, format: "%d %M"},
        {unit: "hour", step: 1, format: "%H:%i"},
      ]
    },
    // Days.
    {
      name: "day",
      scale_height: 27,
      scales: [
        {unit: "day", step: 1, format: "%d %M"}
      ]
    },
    // Weeks.
    {
      name: "week",
      scale_height: 50,
      scales: [
        {
          unit: "week", step: 1, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%d %M");
            var endDate = gantt.date.add(date, -6, "day");
            var weekNum = gantt.date.date_to_str("%W")(date);
            return "#" + weekNum + ", " + dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
        {unit: "day", step: 1, format: "%j %D"}
      ]
    },
    // Months.
    {
      name: "month",
      scale_height: 50,
      scales: [
        {unit: "month", step: 1, format: "%F, %Y"},
        {
          unit: "week", step: 1, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%d %M");
            var endDate = gantt.date.add(gantt.date.add(date, 1, "week"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        }
      ]
    },
    // Quarters.
    {
      name: "quarter",
      height: 50,
      scales: [
        {
          unit: "quarter", step: 3, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%M %y");
            var endDate = gantt.date.add(gantt.date.add(date, 3, "month"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
        {unit: "month", step: 1, format: "%M"},
      ]
    },
    // Years.
    {
      name: "year",
      scale_height: 50,
      scales: [
        {
          unit: "year", step: 5, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%Y");
            var endDate = gantt.date.add(gantt.date.add(date, 5, "year"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        }
      ]
    },
    // Decades.
    {
      name: "year",
      scale_height: 50,
      scales: [
        {
          unit: "year", step: 100, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%Y");
            var endDate = gantt.date.add(gantt.date.add(date, 100, "year"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
        {
          unit: "year", step: 10, format: function (date) {
            var dateToStr = gantt.date.date_to_str("%Y");
            var endDate = gantt.date.add(gantt.date.add(date, 10, "year"), -1, "day");
            return dateToStr(date) + " - " + dateToStr(endDate);
          }
        },
      ]
    },
  ],
  element: function () {
    return gantt.$root.querySelector(".gantt_task");
  }
};

var call_lag_Gantt = {};
var call_lag_gantt = null;

function auto_schedule(e, id) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    if (!call_lag_Gantt[id]) {
      linkLagEditor(Gantt[id], id);
    }
    Gantt[id].config.auto_scheduling = !Gantt[id].config.auto_scheduling;
    if (Gantt[id].config.auto_scheduling) {
      e.classList.add("btn-primary");
    }
    else {
      e.classList.remove("btn-primary");
    }
  }
  else {
    if (!call_lag_gantt) {
      linkLagEditor(gantt);
    }
    gantt.config.auto_scheduling = !gantt.config.auto_scheduling;
    if (gantt.config.auto_scheduling) {
      e.classList.add("btn-primary");
    }
    else {
      e.classList.remove("btn-primary");
    }
  }
}

function linkLagEditor(gantt, id = null) {
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    call_lag_Gantt[id] = true;
  }
  else {
    call_lag_gantt = true;
  }

  function endPopup() {
    modal = null;
    editLinkId = null;
  }

  function cancelEditLink() {
    endPopup()
  }

  function deleteLink() {
    gantt.deleteLink(editLinkId);
    endPopup()
  }

  function saveLink() {
    let link = gantt.getLink(editLinkId);

    let lagValue = modal.querySelector(".lag-input").value;
    if (!isNaN(parseInt(lagValue, 10))) {
      link.lag = parseInt(lagValue, 10);
    }

    gantt.updateLink(link.id);
    if (gantt.autoSchedule) {
      gantt.autoSchedule(link.source);
    }
    endPopup();
  }

  let modal;
  let editLinkId;
  gantt.attachEvent("onLinkDblClick", function (id, e) {
    editLinkId = id;
    let link = gantt.getLink(id);
    if (link.readonly) {
      gantt.message({
        type: 'error',
        text: Drupal.t("You do not have permission to edit this link."),
        expire: 4000
      });
    }
    else {
      let linkTitle;
      switch (link.type) {
        case gantt.config.links.finish_to_start:
          linkTitle = Drupal.t("Finish to start");
          break;

        case gantt.config.links.finish_to_finish:
          linkTitle = Drupal.t("Finish to finish");
          break;

        case gantt.config.links.start_to_start:
          linkTitle = Drupal.t("Start to start");
          break;

        case gantt.config.links.start_to_finish:
          linkTitle = Drupal.t("Start to finish");
          break;
      }

      if (gantt.config.auto_scheduling) {
        modal = gantt.modalbox({
          title: "<div class='fs-5'>" + linkTitle + "</div>",
          text: "<div><b>" + gantt.getTask(link.source).text + "</b> link to <b>" + gantt.getTask(link.target).text + "</b></div><div>" +
            "<label>Lag <input type='number' class='lag-input' /></label>" +
            "</div>",
          buttons: [
            {label: "Save", css: "link-save-btn", value: "save"},
            {label: "Cancel", css: "link-cancel-btn", value: "cancel"},
            {label: "Delete", css: "link-delete-btn", value: "delete"}
          ],
          width: "500px",
          type: "popup-css-class-here",
          callback: function (result) {
            switch (result) {
              case "save":
                saveLink();
                break;

              case "cancel":
                cancelEditLink();
                break;

              case "delete":
                deleteLink();
                break;

            }
          }
        });
        modal.querySelector(".lag-input").value = link.lag || 0;
      }
      else {
        modal = gantt.confirm({
          text: Drupal.t("Link <b>" + gantt.getTask(link.source).text + "</b> - <b>" + gantt.getTask(link.target).text + "</b> will be deleted"),
          ok: Drupal.t("OK"),
          cancel: Drupal.t("Cancel"),
          callback: function (result) {
            if (result) {
              deleteLink();
            }
            else {
              cancelEditLink();
            }
          }
        });
      }
    }
  });
  return true;
}

// Group server list.
function groupGantt(e, id, group_name) {
  let listResource = e.closest('.btn-group-resource').querySelectorAll('.dropdown-item');
  listResource.forEach((element) => {
    element.classList.remove('active');
  });
  e.classList.add("active");
  let listButton = e.closest('.btn-group-resource').children;
  if (e.dataset.field_resource === "default") {
    for (let i = 0; i < listButton.length; i++) {
      if (listButton[i].nodeName == 'BUTTON') {
        listButton[i].classList.remove('btn-primary');
      }
    }
  }
  else {
    for (let i = 0; i < listButton.length; i++) {
      if (listButton[i].nodeName == 'BUTTON') {
        listButton[i].classList.add('btn-primary');
      }
    }
  }
  e.closest('.btn-group-resource').firstElementChild.textContent = e.dataset.label;
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    showGroups(Gantt[id], group_name)
  }
  else {
    showGroups(gantt, group_name)
  }
}

function showGroups(gantt, group_name) {
  if (group_name !== "default") {
    gantt.groupBy({
      groups: gantt.serverList(group_name),
      relation_property: group_name,
      group_id: "key",
      group_text: "label"
    });
  }
  else {
    gantt.groupBy(false);
  }
}

// Search task name.
var filter_data = {};

function change_detector(e, id) {
  filter_data[id] = e.value;
  if (typeof Gantt !== "undefined" && typeof Gantt[id] !== "undefined") {
    Gantt[id].refreshData();
  }
  else {
    gantt.refreshData();
  }
}

function compare_input(gantt, id_gantt, id = 0) {
  let match = false;
  // Check children's text.
  if (gantt.hasChild(id)) {
    gantt.eachTask(function (child_object) {
      if (compare_input(gantt, id_gantt, child_object.id, filter_data[id_gantt])) {
        match = true;
      }
    }, id);
  }

  // Check task's text.
  if (filter_data[id_gantt]) {
    if (gantt.getTask(id).hasOwnProperty('text') && gantt.getTask(id).text.length) {
      if (gantt.getTask(id).text.toLowerCase().indexOf(filter_data[id_gantt].toLowerCase()) >= 0) {
        match = true;
      }
    }
  }
  else {
    return true
  }

  return match;
}
