/**
 * @file
 */

(($, Drupal, cookies) => {
  // Marker today.
  function markerToday(gantt) {
    let dateToStr = gantt.date.date_to_str(gantt.config.task_date);

    let id = gantt.addMarker({
      start_date: new Date(),
      css: "today",
      title: dateToStr(new Date())
    });
    setInterval(function () {
      let today = gantt.getMarker(id);
      if (today) {
        today.start_date = new Date();
        today.title = dateToStr(today.start_date);
        gantt.updateMarker(id);
      }
    }, 1000 * 60);
    // Make resize marker for two columns.
    gantt.attachEvent("onColumnResizeStart", function (ind, column) {
      if (!column.tree || ind == 0) {
        return;
      }

      setTimeout(function () {
        let marker = document.querySelector(".gantt_grid_resize_area");
        if (!marker) {
          return;
        }
        let cols = gantt.getGridColumns();
        let delta = cols[ind - 1].width || 0;
        if (!delta) {
          return;
        }

        marker.style.boxSizing = "content-box";
        marker.style.marginLeft = -delta + "px";
        marker.style.paddingRight = delta + "px";
      }, 1);
    });
  }

  // Undo Redo.
  function undo() {
    gantt.attachEvent("onBeforeAutoSchedule", function () {
      gantt.message("Recalculating project schedule...");
      return true;
    });
    gantt.attachEvent("onAfterTaskAutoSchedule", function (task, new_date, constraint, predecessor) {
      gantt.message({
        text: "<b>" + task.text + "</b> has been rescheduled to " + gantt.templates.task_date(new_date) + " due to <b>" + predecessor.text + "</b> constraint",
        expire: 4000
      });
    });
  }

  // Expand at bottom.
  function expand(gantt) {
    gantt.attachEvent("onCollapse", function () {
      let icon = gantt.toggleIcon;
      if (icon) {
        icon.className = icon.className.replace("fullscreen-exit", "bi-arrows-fullscreen");
      }
    });
    gantt.attachEvent("onExpand", function () {
      let icon = gantt.toggleIcon;
      if (icon) {
        icon.className = icon.className.replace("bi-arrows-fullscreen", "fullscreen-exit");
      }
    });
    gantt.attachEvent("onTemplatesReady", function () {
      let toggle = document.createElement("i");
      toggle.className = "bi bi-arrows-fullscreen gantt-fullscreen";
      gantt.toggleIcon = toggle;
      gantt.$container.appendChild(toggle);
      toggle.onclick = function () {
        gantt.ext.fullscreen.toggle();
      };
    });
  }

  function urlGen(url, param = []) {
    let regex = /-arg_[0-9]-/g;
    let args = url.match(regex);
    args.forEach(function (element, index) {
      url = url.replace(element, param[index])
    })
    return url;
  }

  // User owner.
  function byId(list, id) {
    for (let i = 0; i < list.length; i++) {
      if (list[i].key == parseInt(id)) {
        return list[i].label || "";
      }
    }
    return "";
  }

  function owners(gantt, owner) {
    gantt.form_blocks["multiselect"] = {
      render: function (sns) {
        let height = (sns.height || "23") + "px";
        let html = "<div class='gantt_cal_ltext gantt_cal_chosen gantt_cal_multiselect' style='height:" + height + ";width: 100%'><select data-placeholder='…' class='chosen-select' multiple>";
        if (sns.options) {
          for (let i = 0; i < sns.options.length; i++) {
            if (sns.unassigned_value !== undefined && sns.options[i].key == sns.unassigned_value) {
              continue;
            }
            html += "<option value='" + sns.options[i].key + "'>" + sns.options[i].label + "</option>";
          }
        }
        html += "</select></div>";
        return html;
      },

      set_value: function (node, value, ev, sns) {
        node.style.overflow = "visible";
        node.parentNode.style.overflow = "visible";
        node.style.display = "inline-block";
        let select = $(node.firstChild);

        if (value) {
          value = (value + "").split(",");
          select.val(value);
        }
        else {
          select.val([]);
        }

        select.chosen();
        if (sns.onchange) {
          select.change(function () {
            sns.onchange.call(this);
          })
        }
        select.trigger('chosen:updated');
        select.trigger("change");
      },

      get_value: function (node, ev) {
        let value = $(node.firstChild).val();
        return value;
      },

      focus: function (node) {
        $(node.firstChild).focus();
      }
    };

    gantt.locale.labels.column_owner = Drupal.t("Owner");
    gantt.locale.labels.section_owner = Drupal.t("Owner");
    gantt.config.resource_property = "staff";
    gantt.serverList("staff", owner);

    gantt.attachEvent("onTaskCreated", function (task) {
      task.staff = [];
      task.calendar_id = null;
      return true;
    });
    // Check column owner is existing?
    let isColumnExist = false;
    gantt.config.columns.forEach(function (item) {
      if (item.hasOwnProperty("name")) {
        if (item.name === "owner") {
          isColumnExist = true;
        }
        if (item.name === "text") {
          item.width = 300;
        }
        if (item.name === "duration") {
          item.resize = true;
        }
      }
    });
    if (!isColumnExist) {
      gantt.config.columns.unshift({
        name: "owner", width: 80, align: "center", template: (task) => {
          if (task.type == gantt.config.types.project) {
            return "";
          }
          let result = "";
          let owners = task.owner;
          if (!owners || !owners.length) {
            return "Unassigned";
          }
          if (owners.length == 1) {
            return byId(gantt.serverList('staff'), task.owner[0]);
          }
          owners.forEach((ownerId) => {
            let owner = byId(gantt.serverList('staff'), ownerId);
            if (!owner) {
              return;
            }
            result += "<div class='owner-label' title='" + owner + "'>" + owner.charAt(0).toUpperCase() + "</div>";
          });
          return result;
        }, resize: true
      });

      gantt.config.lightbox.sections.push(
        {
          name: "owner",
          height: 35,
          type: "multiselect",
          options: gantt.serverList("staff"),
          map_to: "owner"
        }
      );
    }

    gantt.templates.grid_row_class =
      gantt.templates.task_row_class =
        gantt.templates.task_class = function (start, end, task) {
          let css = [];
          if (task.$virtual || task.type === gantt.config.types.project) {
            css.push("summary-bar");
          }

          if (task.owner) {
            css.push("gantt_resource_task gantt_resource_" + task.owner);
          }

          return css.join(" ");
        };

    gantt.attachEvent("onDataRender", function () {
      let styleId = "dynamicGanttStyles";
      let element = document.getElementById(styleId);
      if (!element) {
        element = document.createElement("style");
        element.id = styleId;
        document.querySelector("head").appendChild(element);
      }
      let html = [];
      let resources = gantt.serverList("staff");

      resources.forEach(function (r) {
        html.push(".gantt_task_line.gantt_resource_" + r.key + "{" +
          "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
        html.push(".gantt_row.gantt_resource_" + r.key + " .gantt_cell:nth-child(1) .gantt_tree_content{" +
          "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
      });
      element.innerHTML = html.join("");
    });

  }

  function resource(gantt, resource, label, id_resource, show_column = [], show_lightbox = []) {
    gantt.form_blocks["multiselect"] = {
      render: function (sns) {
        let height = (sns.height || "23") + "px";
        let html = "<div class='gantt_cal_ltext gantt_cal_chosen gantt_cal_multiselect' style='min-height:" + height + ";width: 100%'><select data-placeholder='…' class='chosen-select' multiple>";
        if (sns.options) {
          for (let i = 0; i < sns.options.length; i++) {
            if (sns.unassigned_value !== undefined && sns.options[i].key == sns.unassigned_value) {
              continue;
            }
            html += "<option value='" + sns.options[i].key + "'>" + sns.options[i].label + "</option>";
          }
        }
        html += "</select></div>";
        return html;
      },

      set_value: function (node, value, ev, sns) {
        node.style.overflow = "visible";
        node.parentNode.style.overflow = "visible";
        node.style.display = "inline-block";
        let select = $(node.firstChild);

        if (value) {
          value = (value + "").split(",");
          select.val(value);
        }
        else {
          select.val([]);
        }

        select.chosen();
        if (sns.onchange) {
          select.change(function () {
            sns.onchange.call(this);
          })
        }
        select.trigger('chosen:updated');
        select.trigger("change");
      },

      get_value: function (node, ev) {
        let value = $(node.firstChild).val();
        return value;
      },

      focus: function (node) {
        $(node.firstChild).focus();
      }
    };

    gantt.locale.labels["section_" + id_resource] = Drupal.t(label);
    gantt.locale.labels["column_" + id_resource] = Drupal.t(label);
    gantt.config.resource_property = id_resource;
    gantt.serverList(id_resource, resource);

    gantt.attachEvent("onTaskCreated", function (task) {
      task.staff = [];
      task.calendar_id = null;
      return true;
    });
    // Check column is existing?
    let colExists = gantt.config.columns.findIndex((element) => element.name == id_resource);

    // Show column.
    if (show_column.includes(id_resource) && colExists < 0) {
      gantt.config.columns.unshift({
        name: id_resource, width: 80, align: "center", template: (task) => {
          if (task.type == gantt.config.types.project) {
            return "";
          }
          let result = "";
          let resources = task[id_resource];
          if (!resources || !resources.length) {
            return "Unassigned";
          }
          if (resources.length == 1) {
            return byId(gantt.serverList(id_resource), task[id_resource][0]);
          }
          resources.forEach((resourceId) => {
            let resource_name = byId(gantt.serverList(id_resource), resourceId);
            if (!resource_name) {
              return;
            }
            result += "<div class='owner-label' title='" + resource_name + "'>" + resource_name.charAt(0).toUpperCase() + "</div>";
          });
          return result;
        }, resize: true
      });
    }

    // Show in light box.
    if (show_lightbox.includes(id_resource)) {
      gantt.config.lightbox.sections.push(
        {
          name: id_resource,
          height: 35,
          type: "multiselect",
          options: gantt.serverList(id_resource),
          map_to: id_resource
        }
      );
    }

    gantt.attachEvent("onDataRender", function () {
      let styleId = "dynamicGanttStyles";
      let element = document.getElementById(styleId);
      if (!element) {
        element = document.createElement("style");
        element.id = styleId;
        document.querySelector("head").appendChild(element);
      }
      let html = [];
      let resources = gantt.serverList(id_resource);

      resources.forEach(function (r) {
        html.push(".gantt_task_line.gantt_resource_" + r.key + "{" +
          "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
        html.push(".gantt_row.gantt_resource_" + r.key + " .gantt_cell:nth-child(1) .gantt_tree_content{" +
          "background-color:" + r.backgroundColor + "; " + "color:" + r.textColor + ";" + "}");
      });
      element.innerHTML = html.join("");
    });

  }

  // Lightbox.
  function setLightBox(gantt, setting) {
    let formatter = gantt.ext.formatters.durationFormatter({
      enter: "day",
      store: "hour",
      format: "day",
      hoursPerDay: 24,
      hoursPerWeek: 40,
      daysPerMonth: 30,
      short: true
    });

    // Light box task.
    gantt.config.lightbox.sections = [
      {
        name: "description",
        height: 70,
        map_to: "text",
        type: "textarea",
        focus: true
      },
      {
        name: "type",
        height: "auto",
        type: "typeselect",
        map_to: "type",
        filter: function (name, value) {
          return !!(value != gantt.config.types.project);
        }
      },
      {name: "time", type: "duration", map_to: "auto"}
    ];

    if (setting.select_parent) {
      let position = gantt.config.lightbox.project_sections.findIndex((element) => element.name == "type")
      gantt.config.lightbox.sections.splice(position + 1, 0, {
        name: "parent",
        type: "parent",
        allow_root: "true",
        root_label: "No parent",
        filter: function (id, task) {
          return true;
        }
      })
    }

    if (!setting.use_cnd) {
      let position = gantt.config.lightbox.sections.findIndex((element) => element.name == "time")
      gantt.config.lightbox.sections.splice(position + 1, 0, {
        name: "baseline",
        button: true,
        type: "duration_optional",
        map_to: {start_date: "planned_start", end_date: "planned_end"}
      })
    }

    // Light box project.
    gantt.config.lightbox.project_sections = [
      {
        name: "description",
        height: 70,
        map_to: "text",
        type: "textarea",
        focus: true
      },
      {name: "time", type: "duration", readonly: true, map_to: "auto"}
    ];

    if (setting.select_parent) {
      let position = gantt.config.lightbox.project_sections.findIndex((element) => element.name == "type")
      gantt.config.lightbox.project_sections.splice(position + 1, 0, {
        name: "parent",
        type: "parent",
        allow_root: "true",
        root_label: "No parent",
        filter: function (id, task) {
          return true;
        }
      })
    }

    if(!setting.use_cnd) {
      let position = gantt.config.lightbox.project_sections.findIndex((element) => element.name == "time")
      gantt.config.lightbox.project_sections.splice(position + 1, 0, {
        name: "baseline",
        type: "duration_optional",
        map_to: {start_date: "planned_start", end_date: "planned_end"}
      })
    }

    // Light box milestone.
    gantt.config.lightbox.milestone_sections = [
      {
        name: "description",
        height: 70,
        map_to: "text",
        type: "textarea",
        focus: true
      },
      {
        name: "type",
        height: "auto",
        type: "typeselect",
        map_to: "type",
        filter: function (name, value) {
          return !!(value != gantt.config.types.project);
        }
      },
      {
        name: "parent",
        type: "parent",
        allow_root: "true",
        root_label: "No parent",
        filter: function (id, task) {
          if (task.$level > 1) {
            return false;
          }
          else {
            return true;
          }
        }
      },
      {
        name: "time",
        type: "duration",
        single_date: true,
        map_to: "auto",
        formatter: formatter
      }
    ];

  }

  // Tooltip.
  function setTooltip(gantt, setting) {
    // Set format date tooltips.
    gantt.templates.tooltip_date_format = function (date) {
      let formatFunc = gantt.date.date_to_str("%d-%m-%Y");
      return formatFunc(date);
    };

    // Setup tooltip content.
    gantt.templates.tooltip_text = function (start, end, task) {
      let text_end = gantt.templates.tooltip_date_format(new Date(end.valueOf()));
      // Option Last of the day.
      if (setting.last_of_the_day) {
        text_end = gantt.templates.tooltip_date_format(new Date(end.valueOf() - 1));
      }
      let creator = Drupal.t("Unassigned");
      if (typeof task != 'undefined' && task.hasOwnProperty('creator_name')) {
        if (task.creator_name.length) {
          creator = task.creator_name.join(", ")
        }
      }
      let html = "<b>" + Drupal.t('Task') + ":</b> " + task.text +
        "<br/><b>" + Drupal.t("Start") + ":</b> " + gantt.templates.tooltip_date_format(start) +
        "<br/><b>" + Drupal.t("End") + ":</b> " + text_end +
        "<br><b>" + Drupal.t("Duration") + ":</b> " + task.duration + Drupal.t(' days') +
        "<br><b>" + Drupal.t("Creator") + ":</b> " + creator;
      return html;
    };
  }

  function upload(gantt, file, callback, urlImport) {
    gantt.importFromMSProject({
      data: file,
      callback: function (project) {
        if (project) {
          gantt.clearAll();
          if (project.config.duration_unit) {
            gantt.config.duration_unit = project.config.duration_unit;
          }
          // mark tasks import.
          for (let i = 0; i < project.data.data.length; i++) {
            project.data.data[i].isImport = true;
          }
          gantt.parse(project.data);
          $.post(urlImport, project.data).done(function (data) {
            console.log("Data import: " + data);
          });
          if (callback) {
            callback(project);
          }
        }
      }
    });
  }

  function importMMP(gantt, ganttId, urlImport) {
    // Import microsoft project.
    let fileDnD = fileDragAndDrop();
    fileDnD.init(gantt.$container);

    function sendFile(file) {
      fileDnD.showUpload();
      upload(gantt, file, function () {
        fileDnD.hideOverlay();
      }, urlImport)
    }

    fileDnD.onDrop(sendFile);
    // Manual Upload file MPP.
    $("#mspImportBtn" + ganttId).click(function (event) {
      event.preventDefault();
      let fileInput = document.getElementById("mspFile" + ganttId);
      if (fileInput.files[0]) {
        sendFile(fileInput.files[0]);
      }
    });
  }

  function native_dialog(gantt, setting) {
    gantt.attachEvent("onTaskDblClick", function (id, e) {
      if (id == null) {
        return false;
      }
      let task = gantt.getTask(id);
      if (task['type'] == 'project') {
        return false;
      }
      else {
        let url = task.link_detail;
        if (url == undefined) {
          url = setting.link_detail.edit;
          if (task.hasOwnProperty("readonly")) {
            url = setting.link_detail.display;
          }
          url = urlGen(url, [id]);
        }
        let ajaxSettings = {
          url: url,
          dialogType: 'modal',
          dialog: {width: '80%'},
        };
        Drupal.ajax(ajaxSettings).execute();
        return false;
      }
    });

    gantt.attachEvent("onBeforeLightbox", function (id) {
      let task = gantt.getTask(id);
      if (task == null) {
        return false;
      }
      if (task.$new) {
        if (setting.add_task !== true) {
          gantt.deleteTask(task.id);
          return false;
        }
        let add_link = setting.add_link + '&parent=' + task.parent + '&parent_field=' + setting.parent_field;
        let ajaxSettings = {
          url: add_link,
          dialogType: 'modal',
          dialog: {width: '80%'},
        };
        Drupal.ajax(ajaxSettings).execute();
        gantt.deleteTask(task.id);
        return false;
      }
      return true;
    });

  }

  // Recalculate progress of summary tasks when the progress of subtasks changes
  function dynamicProgress(gantt) {

    function calculateSummaryProgress(task) {
      if (task.type != gantt.config.types.project)
        return task.progress;
      let totalToDo = 0;
      let totalDone = 0;
      gantt.eachTask(function (child) {
        if (child.type != gantt.config.types.project) {
          totalToDo += child.duration;
          totalDone += (child.progress || 0) * child.duration;
        }
      }, task.id);
      if (!totalToDo) return 0;
      else return totalDone / totalToDo;
    }

    function refreshSummaryProgress(id, submit) {
      if (!gantt.isTaskExists(id))
        return;

      let task = gantt.getTask(id);
      let newProgress = calculateSummaryProgress(task);

      if (newProgress !== task.progress) {
        task.progress = newProgress;

        if (!submit) {
          gantt.refreshTask(id);
        } else {
          gantt.updateTask(id);
        }
      }

      if (!submit && gantt.getParent(id) !== gantt.config.root_id) {
        refreshSummaryProgress(gantt.getParent(id), submit);
      }
    }

    gantt.attachEvent("onParse", function () {
      gantt.eachTask(function (task) {
        task.progress = calculateSummaryProgress(task);
      });
    });

    gantt.attachEvent("onAfterTaskUpdate", function (id) {
      refreshSummaryProgress(gantt.getParent(id), true);
    });

    gantt.attachEvent("onTaskDrag", function (id) {
      refreshSummaryProgress(gantt.getParent(id), false);
    });
    gantt.attachEvent("onAfterTaskAdd", function (id) {
      refreshSummaryProgress(gantt.getParent(id), true);
    });

    (function () {
      let idParentBeforeDeleteTask = 0;
      gantt.attachEvent("onBeforeTaskDelete", function (id) {
        idParentBeforeDeleteTask = gantt.getParent(id);
      });
      gantt.attachEvent("onAfterTaskDelete", function () {
        refreshSummaryProgress(idParentBeforeDeleteTask, true);
      });
    })();

    gantt.attachEvent("onBeforeTaskChanged", function (id, mode, old_event) {
      let task = gantt.getTask(id);
      if (mode == gantt.config.drag_mode.progress) {
        if (task.type == gantt.config.types.project) {
          gantt.message(task.text + " progress can't be undone!");
          return false;
        }
      }
      return true;
    });
  }

  let plugins = {
    fullscreen: true,
    marker: true,
    critical_path: true,
    tooltip: true,
    multiselect: true,
    auto_scheduling: true,
    drag_timeline: true,
    grouping: true,
    overlay: true,
    undo: true,
  };
  let i18n = {
    date: {
      month_full: [Drupal.t("January"), Drupal.t("February"), Drupal.t("March"), Drupal.t("April"), Drupal.t("May"), Drupal.t("June"),
        Drupal.t("July"), Drupal.t("August"), Drupal.t("September"), Drupal.t("October"), Drupal.t("November"), Drupal.t("December")],
      month_short: [Drupal.t("Jan"), Drupal.t("Feb"), Drupal.t("Mar"), Drupal.t("Apr"), Drupal.t("May"), Drupal.t("Jun"), Drupal.t("Jul"),
        Drupal.t("Aug"), Drupal.t("Sep"), Drupal.t("Oct"), Drupal.t("Nov"), Drupal.t("Dec")],
      day_full: [Drupal.t("Sunday"), Drupal.t("Monday"), Drupal.t("Tuesday"), Drupal.t("Wednesday"), Drupal.t("Thursday"), Drupal.t("Friday"), Drupal.t("Saturday")],
      day_short: [Drupal.t("Sun"), Drupal.t("Mon"), Drupal.t("Tue"), Drupal.t("Wed"), Drupal.t("Thu"), Drupal.t("Fri"), Drupal.t("Sat")]
    },
    labels: {
      new_task: Drupal.t("New task"),
      icon_save: Drupal.t("Save"),
      icon_cancel: Drupal.t("Cancel"),
      icon_details: Drupal.t("Details"),
      icon_edit: Drupal.t("Edit"),
      icon_delete: Drupal.t("Delete"),
      gantt_save_btn: Drupal.t("New Label"),
      gantt_cancel_btn: Drupal.t("New Label"),
      gantt_delete_btn: Drupal.t("New Label"),
      confirm_deleting: Drupal.t("Task will be deleted permanently, are you sure?"),
      section_description: Drupal.t("Description"),
      section_time: Drupal.t("Time period"),
      section_type: Drupal.t("Type"),
      details_btn: Drupal.t("Details"),
      section_parent: Drupal.t("Parent task"),
      // **
      section_field_reference_task_kanban: Drupal.t("Link task to kanban"),

      // Setup lightbox.
      section_baseline: Drupal.t("Planned"),
      baseline_enable_button: Drupal.t('Set'),
      baseline_disable_button: Drupal.t('Remove'),

      /* grid columns */
      column_wbs: "WBS",
      column_text: Drupal.t("Task name"),
      column_start_date: Drupal.t("Start time"),
      column_duration: Drupal.t("Duration"),

      /* link confirmation */
      link: Drupal.t("Link"),
      confirm_link_deleting: Drupal.t("will be deleted"),
      link_start: Drupal.t(" (start)"),
      link_end: Drupal.t(" (end)"),

      type_task: Drupal.t("Task"),
      type_project: Drupal.t("Project"),
      type_milestone: Drupal.t("Milestone"),
      type_placeholder: Drupal.t("Placeholder"),

      minutes: Drupal.t("Minutes"),
      hours: Drupal.t("Hours"),
      days: Drupal.t("Days"),
      weeks: Drupal.t("Week"),
      months: Drupal.t("Months"),
      years: Drupal.t("Years"),

      /* message popup */
      message_ok: Drupal.t("OK"),
      message_cancel: Drupal.t("Cancel"),

      /* constraints */
      section_constraint: Drupal.t("Constraint"),
      constraint_type: Drupal.t("Constraint type"),
      constraint_date: Drupal.t("Constraint date"),
      asap: Drupal.t("As Soon As Possible"),
      alap: Drupal.t("As Late As Possible"),
      snet: Drupal.t("Start No Earlier Than"),
      snlt: Drupal.t("Start No Later Than"),
      fnet: Drupal.t("Finish No Earlier Than"),
      fnlt: Drupal.t("Finish No Later Than"),
      mso: Drupal.t("Must Start On"),
      mfo: Drupal.t("Must Finish On"),

      /* resource control */
      resources_filter_placeholder: Drupal.t("type to filter"),
      resources_filter_label: Drupal.t("hide empty")
    }
  };
  Drupal.behaviors.paragraphs_gantt = {
    attach: (context, settings) => {
      $(once('paragraphs_gantt', 'body', context)).each(function () {
        if (settings.gantt) {
          $.each(settings.gantt, function (key, ganttField) {
            // Format plan date.
            if (dataGantt[ganttField.id].data) {
              dataGantt[ganttField.id].data.forEach(function (itemGantt, index) {
                if (itemGantt?.planned_start !== undefined && itemGantt?.planned_end !== undefined) {
                  itemGantt.planned_start = itemGantt.planned_start.concat("T00:00:00");
                  itemGantt.planned_end = itemGantt.planned_end.concat("T00:00:00");
                }
              });
            }

            // Ajax link action.
            let configAjax = {
              url: ganttField.ajax,
              mode: "POST"
            };

            if (Object.values(settings.gantt).length == 1 && !ganttField.split_actual_planned) {

              gantt.$zoomToFit = false;
              gantt.i18n.setLocale(i18n);
              gantt.plugins(plugins);

              // Set config gantt.
              gantt.config.auto_types = true;
              gantt.config.xml_date = "%Y-%m-%d";
              gantt.config.date_format = "%Y-%m-%d";
              gantt.config.date_grid = settings.date_format;
              gantt.config.duration_unit = "day";
              gantt.config.duration_step = 1;
              gantt.config.order_branch = "marker";
              gantt.config.sort = true;
              gantt.config.fit_tasks = true;
              gantt.config.order_branch_free = true;
              gantt.config.grid_resize = true;
              gantt.config.server_utc = false;
              gantt.config.work_time = false;
              gantt.config.reorder_grid_columns = true;
              gantt.config.open_tree_initially = true;

              markerToday(gantt);
              expand(gantt);
              setLightBox(gantt, ganttField);
              setTooltip(gantt, ganttField);

              // Recalculate progress of summary.
              if (ganttField.dynamic_progress) {
                dynamicProgress(gantt);
              }

              // Css class.
              gantt.templates.grid_row_class = function (start, end, task) {
                let css = [];
                if (task.$virtual || task.type === gantt.config.types.project) {
                  css.push("summary-row");
                }
                if (task.progress >= 1) {
                  css.push("summary-grid-row-green");
                }
                if (task.owner) {
                  css.push("gantt_resource_task gantt_resource_" + task.owner);
                }

                return css.join(" ");
              };

              gantt.templates.task_row_class = function (start, end, task) {
                let css = [];
                if (task.$virtual || task.type === gantt.config.types.project) {
                  css.push("summary-row");
                }
                if (task.progress >= 1) {
                  css.push("summary-task-row-green");
                }
                if (task.owner) {
                  css.push("gantt_resource_task gantt_resource_" + task.owner);
                }

                return css.join(" ");
              };

              gantt.templates.task_class = function (start, end, task) {
                let css = [];
                if (task.$virtual || task.type === gantt.config.types.project) {
                  css.push("summary-bar");
                }
                /* Add class task when exist base line. */
                if (ganttField.has_planned && !ganttField.split_actual_planned && !ganttField.use_cdn) {
                  css.push("exist_base_line");

                  /* Add class Overdue. */
                  if (task.planned_end) {
                    css.push("has-baseline");
                    if (end.getTime() > task.planned_end.getTime()) {
                      css.push('overdue');
                    }
                  }
                }
                return css.join(" ");
              };

              // Drag timeline.
              gantt.config.drag_timeline = {
                ignore: ".gantt_task_line, .gantt_task_link",
                useKey: "ctrlKey"
              };

              gantt.attachEvent("onBeforeTaskDrag", function (id, mode, e) {
                if (e.ctrlKey) {
                  return false;
                }
                return true;
              });

              // Set width text column.
              let colText = gantt.config.columns.findIndex((element) => element.name == "text");
              if (colText >= 0) {
                gantt.config.columns[colText].min_width = 200;
              }

              // Set resize duration column.
              let colDuration = gantt.config.columns.findIndex((element) => element.name == "duration");
              if (colDuration >= 0) {
                gantt.config.columns[colDuration].resize = true;
              }

              // Custom button.
              if (ganttField.show_button_detail) {
                gantt.attachEvent("onGanttReady", function (task) {
                  gantt.config.buttons_left = ["gantt_save_btn", "details_btn", "gantt_cancel_btn"];
                  gantt.config.buttons_right = ["gantt_delete_btn"];
                });

                gantt.attachEvent("onLightboxButton", function (button_id, node, e, task) {
                  if (button_id == "details_btn") {
                    let id = gantt.getState().lightbox;
                    let task = gantt.getTask(id);
                    if (!task.$new) {
                      let url = task.link_detail;
                      if (url == undefined) {
                        url = ganttField.link_detail.edit;
                        if (task.hasOwnProperty("readonly")) {
                          url = ganttField.link_detail.display;
                        }
                        url = urlGen(url, [id]);
                      }

                      let ajaxSettings = {
                        url: url,
                        dialogType: 'modal',
                        dialog: {width: '80%'},
                      };
                      Drupal.ajax(ajaxSettings).execute();
                      gantt.hideLightbox();
                      return false;
                    }
                  }
                });
              }

              // Button action.
              gantt.attachEvent("onLightboxSave", function (id, task, is_new) {
                if (gantt.isReadonly(id)) {
                  gantt.alert(Drupal.t("You cannot edit this task."));
                  return false;
                }
                if (task.text.length > 255) {
                  gantt.alert(Drupal.t("Description cannot be too long"));
                  return false;
                }
                return true;
              })

              gantt.attachEvent("onLightboxDelete", function (id, task, is_new) {
                if (gantt.isReadonly(id)) {
                  gantt.alert(Drupal.t("You cannot delete this task."));
                  return false;
                }
                return true;
              })

              // Show progress %.
              if (ganttField.has_duration) {
                gantt.templates.progress_text = function (start, end, task) {
                  return "<span style='text-align:left;'>" + Math.round(task.progress * 100) + "% </span>";
                };
              }

              // Add column +.
              if (!ganttField.add_task) {
                $.each(gantt.config.columns, function (index, col) {
                  if (col.name === 'add') {
                    delete col.name;
                  }
                });
              }

              // Change mode view readonly.
              if (ganttField.edit_task === false) {
                gantt.config.readonly = true;
              }

              // Switch dialog native drupal.
              if (ganttField.native_dialog) {
                native_dialog(gantt, ganttField);
              }

              // Add custom column custom field.
              if (ganttField.custom_field) {
                let customColumnExist = gantt.config.columns.findIndex((element) => element.name == ganttField.custom_field);
                if (customColumnExist < 0) {
                  gantt.config.columns.splice(gantt.config.columns.length - 1, 0, {
                    name: ganttField.custom_field,
                    label: ganttField.custom_field_name,
                    align: "center",
                    resize: true
                  });
                  gantt.locale.labels['section_' + ganttField.custom_field] = ganttField.custom_field_name;
                }
                let customLightBoxExist = gantt.config.lightbox.sections.findIndex((element) => element.name == ganttField.custom_field);
                if (customLightBoxExist < 0) {
                  gantt.config.lightbox.sections.push(
                    {
                      name: ganttField.custom_field,
                      type: "number",
                      map_to: ganttField.custom_field,
                    }
                  );
                  gantt.form_blocks["number"] = {
                    render: function (sns) {
                      return '<input class="form-control"  type="number" min="0">';
                    },
                    set_value: function (node, value, task) {
                      node.value = value;
                    },
                    get_value: function (node, task) {
                      return node.value || task.options;
                    },
                    focus: function (node) {
                    }
                  };
                }
              }

              // Add baseline on row.
              // Show 2nd chart for actual date or baseline.
              if (ganttField.has_planned && !ganttField.split_actual_planned && !ganttField.use_cdn) {
                gantt.config.bar_height = 16;
                gantt.config.row_height = 40;

                /* Adding baseline display. */
                if (gantt.hasOwnProperty('addTaskLayer')) {
                  gantt.addTaskLayer({
                    renderer: {
                      render: function draw_planned(task) {
                        /* Add class link when exist base line. */
                        gantt.templates.link_class = function (link) {
                          return "exist_base_line_link";
                        };
                        /* Add base line. */
                        if (task.planned_start && task.planned_end) {
                          let sizes = gantt.getTaskPosition(task, task.planned_start, task.planned_end);
                          let el = document.createElement('div');
                          el.className = 'baseline';
                          el.style.left = sizes.left + 'px';
                          el.style.width = sizes.width + 'px';
                          el.style.top = sizes.top + gantt.config.bar_height + 13 + 'px';
                          return el;
                        }
                        return false;
                      },
                      /* Define getRectangle in order to hook layer with the. */
                      getRectangle: function (task, view) {
                        if (task.planned_start && task.planned_end) {
                          return gantt.getTaskPosition(task, task.planned_start, task.planned_end);
                        }
                        return null;
                      }
                    }
                  });

                  /* Calculator Overdue. */
                  gantt.templates.rightside_text = function (start, end, task) {
                    if (task.planned_end) {
                      if (end.getTime() > task.planned_end.getTime()) {
                        let overdue = Math.ceil(Math.abs((end.getTime() - task.planned_end.getTime()) / (24 * 60 * 60 * 1000)));
                        let text = `<b>${Drupal.t('Overdue')}: ${overdue} ${Drupal.t('days')}</b>`;
                        return text;
                      }
                    }
                  };

                  gantt.attachEvent("onTaskLoading", function (task) {
                    task.planned_start = gantt.date.parseDate(task.planned_start, "xml_date");
                    task.planned_end = gantt.date.parseDate(task.planned_end, "xml_date");
                    return true;
                  });
                }
              }

              // Add workTime and holiday.
              if (ganttField.work_time) {
                gantt.config.work_time = true;
                /* Add holiday. */
                if (ganttField.holidays) {
                  let holidays = ganttField.holidays.map(dateString => {
                    let dateParts = dateString.split('-');
                    let year = parseInt(dateParts[0]);
                    let month = parseInt(dateParts[1]) - 1;
                    let day = parseInt(dateParts[2]);
                    gantt.setWorkTime({
                      date: new Date(year, month, day),
                      hours: false
                    });
                    return new Date(year, month, day);
                  });
                }

                gantt.setWorkTime({day: 0, hours: true});
                gantt.setWorkTime({day: 6, hours: true});
                if (ganttField.work_day.length) {
                  const workDays = ganttField.work_day.map(day => {
                    gantt.setWorkTime({day: day, hours: false});
                    return day;
                  });
                }
                gantt.templates.scale_cell_class = function (date) {
                  if (!gantt.isWorkTime(date)) {
                    return "weekend";
                  }
                };
                gantt.templates.timeline_cell_class = function (task, date) {
                  if (!gantt.isWorkTime({task: task, date: date})) {
                    return "weekend";
                  }
                  return "";
                };
              }

              // Set zoom config for charts.
              let zoomConfig = {
                minColumnWidth: 50,
                maxColumnWidth: 150,
                levels: [
                  {
                    name: "year",
                    scales: [
                      {unit: "year", format: "%Y", step: 1},
                    ]
                  },
                  {
                    name: "quarter",
                    scales: [
                      {unit: "year", format: "%Y", step: 1},
                      {
                        unit: "quarter", step: 1, format: function (date) {
                          let dateToStr = gantt.date.date_to_str("%M");
                          let endDate = gantt.date.add(gantt.date.add(date, 3, "month"), -1, "day");
                          return dateToStr(date) + " - " + dateToStr(endDate);
                        }
                      }
                    ]
                  },
                  {
                    name: "month",
                    scales: [
                      {unit: "year", format: "%Y", step: 1},
                      {unit: "month", format: "%M", step: 1}
                    ]
                  },
                  {
                    name: "week",
                    scales: [
                      {unit: "month", format: "%M %Y", step: 1},
                      {
                        unit: "week", step: 1, format: function (date) {
                          let dateToStr = gantt.date.date_to_str("%d/%m");
                          let endDate = gantt.date.add(gantt.date.add(date, 7, "day"), -1, "day");
                          let weekNum = gantt.date.date_to_str("%W")(date);
                          return "#" + weekNum + ", " + dateToStr(date) + " - " + dateToStr(endDate);
                        }
                      }
                    ]
                  },
                  {
                    name: "day",
                    scales: [
                      {unit: "month", format: "%M %Y", step: 1},
                      {unit: "day", format: "%d %M", step: 1}
                    ]
                  }
                ],
                useKey: "ctrlKey",
                trigger: "wheel",
                element: function () {
                  return gantt.$root.querySelector(".gantt_task");
                }
              }

              gantt.ext.zoom.init(zoomConfig);
              gantt.ext.zoom.setLevel("day");
              gantt.ext.zoom.attachEvent("onAfterZoom", function (level, config) {
                document.querySelector('[onchange^="set_scale"]').value = config.name;
              })

              // Add show column end date.
              if (ganttField.show_end) {
                let indexCol = gantt.config.columns.findIndex((item) => item.name == 'start_date');
                gantt.config.columns.splice(indexCol + 1, 0, {
                  name: "end_date",
                  width: 80,
                  label: Drupal.t("End"),
                  align: "center",
                  resize: true
                });
              }

              // Owner.
              if (ganttField.owner) {
                owners(gantt, ownerGantt[ganttField.id]);
              }

              // Set resource.
              if (ganttField.custom_resource.length) {
                ganttField.custom_resource.map(function (id_resource, index) {
                  resource(gantt, resourceGantt[ganttField.id][id_resource], ganttField['setting_resource']['resource_label'][id_resource], id_resource, ganttField['setting_resource']['resource_column'], ganttField['setting_resource']['resource_lightbox']);
                })
              }

              // Only links.
              gantt.attachEvent("onBeforeLinkAdd", function (id, link) {
                let taskSource = gantt.getTask(link.source);
                let taskTarget = gantt.getTask(link.target);
                if (gantt.isReadonly(link.target)) {
                  gantt.message({
                    type: "error",
                    text: Drupal.t("Can not link. <b>" + taskTarget.text + "</b> not permission."),
                    expire: 4000
                  });
                  return false;
                }
                if (taskSource.$source.includes(link.source + "-" + link.target) || taskTarget.$source.includes(link.target + "-" + link.source)) {
                  gantt.message({
                    type: "error",
                    text: Drupal.t("Link <b>" + taskSource.text + "</b> to <b>" + taskTarget.text + "</b> already exist."),
                    expire: 4000
                  });
                  return false;
                }
                return true;
              });

              // Automatic scheduling.
              if (ganttField.auto_schedule && !ganttField.use_cdn) {
                linkLagEditor(gantt);

                gantt.config.auto_scheduling = true;
                gantt.attachEvent("onAfterTaskAutoSchedule", function (task, new_date, constraint, predecessor) {
                  if (task && predecessor) {
                    gantt.message({
                      text: Drupal.t("<b>" + task.text + "</b> has been rescheduled to " + gantt.templates.task_date(new_date) + " due to <b>" + predecessor.text + "</b> constraint"),
                      expire: 4000
                    });
                  }
                });
              }

              // Display task search text.
              gantt.attachEvent("onBeforeTaskDisplay", function (id, task) {
                if (compare_input(gantt, key, id)) {
                  return true;
                }
                return false;
              });

              // Format end date.
              gantt.templates.task_end_date = function (date) {
                // Option Last of the day.
                if (ganttField.last_of_the_day) {
                  return gantt.templates.task_date(new Date(date.valueOf() - 1));
                }
                return gantt.templates.task_date(new Date(date.valueOf()));
              };
              let gridDateToStr = gantt.date.date_to_str("%d-%m-%Y");
              gantt.templates.grid_date_format = function (date, column) {
                if (column === "end_date") {
                  // Option Last of the day.
                  if (ganttField.last_of_the_day) {
                    return gridDateToStr(new Date(date.valueOf() - 1));
                  }
                  return gridDateToStr(new Date(date.valueOf()));
                }
                else {
                  return gridDateToStr(date);
                }
              }

              // Error ajax.
              gantt.attachEvent("onAjaxError", function (request) {
                gantt.message({
                  type: "error",
                  text: "Error " + request.status + "!",
                  expire: 4000
                })
                return true;
              });

              // Trigger gantt for mono chart.
              gantt.createDataProcessor(configAjax);
              if ($('#' + ganttField.id).length) {
                gantt.init(ganttField.id);
              }
              if (typeof dataGantt != "undefined" && dataGantt[ganttField.id] && dataGantt[ganttField.id].data !== undefined) {
                gantt.parse(dataGantt[ganttField.id]);
              }

              importMMP(gantt, ganttField.id, ganttField.import);
            }
            else if (typeof Gantt !== "undefined") {

              // Trigger gantt for multi charts.
              Gantt[key] = Gantt.getGanttInstance();
              Gantt[key].i18n.setLocale(i18n);
              Gantt[key].plugins(plugins);

              // Set config gantt.
              Gantt[key].config.auto_types = true;
              Gantt[key].config.xml_date = "%Y-%m-%d";
              Gantt[key].config.date_format = "%Y-%m-%d";
              Gantt[key].config.date_grid = settings.date_format;
              Gantt[key].config.duration_unit = "day";
              Gantt[key].config.duration_step = 1;
              Gantt[key].config.order_branch = "marker";
              Gantt[key].config.sort = true;
              Gantt[key].config.fit_tasks = true;
              Gantt[key].config.order_branch_free = true;
              Gantt[key].config.grid_resize = true;
              Gantt[key].config.server_utc = false;
              Gantt[key].config.work_time = false;
              Gantt[key].config.reorder_grid_columns = true;

              markerToday(Gantt[key]);
              expand(Gantt[key]);
              setLightBox(Gantt[key], ganttField);
              setTooltip(Gantt[key], ganttField);

              // Recalculate progress of summary.
              if (ganttField.dynamic_progress) {
                dynamicProgress(Gantt[key]);
              }

              // Css class.
              Gantt[key].templates.grid_row_class = function (start, end, task) {
                let css = [];
                if (task.$virtual || task.type === Gantt[key].config.types.project) {
                  css.push("summary-row");
                }
                if (task.progress >= 1) {
                  css.push("summary-grid-row-green");
                }
                if (task.owner) {
                  css.push("gantt_resource_task gantt_resource_" + task.owner);
                }

                return css.join(" ");
              };

              Gantt[key].templates.task_row_class = function (start, end, task) {
                let css = [];
                if (task.$virtual || task.type === Gantt[key].config.types.project) {
                  css.push("summary-row");
                }
                if (task.progress >= 1) {
                  css.push("summary-task-row-green");
                }
                if (task.owner) {
                  css.push("gantt_resource_task gantt_resource_" + task.owner);
                }

                return css.join(" ");
              };

              Gantt[key].templates.task_class = function (start, end, task) {
                let css = [];
                if (task.$virtual || task.type === Gantt[key].config.types.project) {
                  css.push("summary-bar");
                }
                /* Add class task when exist base line. */
                if (ganttField.has_planned && !ganttField.split_actual_planned && !ganttField.use_cdn) {
                  css.push("exist_base_line");

                  /* Add class Overdue. */
                  if (task.planned_end) {
                    css.push("has-baseline");
                    if (end.getTime() > task.planned_end.getTime()) {
                      css.push('overdue');
                    }
                  }
                }
                return css.join(" ");
              };

              // Drag timeline.
              Gantt[key].config.drag_timeline = {
                ignore: ".gantt_task_line, .gantt_task_link",
                useKey: "ctrlKey"
              };

              Gantt[key].attachEvent("onBeforeTaskDrag", function (id, mode, e) {
                if (e.ctrlKey) {
                  return false;
                }
                return true;
              });

              // Set width text column.
              let colText = Gantt[key].config.columns.findIndex((element) => element.name == "text");
              if (colText >= 0) {
                Gantt[key].config.columns[colText].min_width = 200;
              }

              // Set resize duration column.
              let colDuration = Gantt[key].config.columns.findIndex((element) => element.name == "duration");
              if (colDuration >= 0) {
                Gantt[key].config.columns[colDuration].resize = true;
              }

              // Custom button.
              if (ganttField.show_button_detail) {
                Gantt[key].attachEvent("onGanttReady", function () {
                  Gantt[key].config.buttons_left = ["gantt_save_btn", "details_btn", "gantt_cancel_btn"];
                  Gantt[key].config.buttons_right = ["gantt_delete_btn"];
                });

                Gantt[key].attachEvent("onLightboxButton", function (button_id, node, e, task) {
                  if (button_id == "details_btn") {
                    let id = Gantt[key].getState().lightbox;
                    let task = Gantt[key].getTask(id);
                    if (!task.$new) {
                      let url = task.link_detail;
                      if (url == undefined) {
                        url = ganttField.link_detail.edit;
                        if (task.hasOwnProperty("readonly")) {
                          url = ganttField.link_detail.display;
                        }
                        url = urlGen(url, [id]);
                      }
                      let ajaxSettings = {
                        url: url,
                        dialogType: 'modal',
                        dialog: {width: '80%'},
                      };
                      Drupal.ajax(ajaxSettings).execute();
                      Gantt[key].hideLightbox();
                      return false;
                    }
                  }
                });
              }

              // Button action.
              Gantt[key].attachEvent("onLightboxSave", function (id, task, is_new) {
                if (Gantt[key].isReadonly(id)) {
                  Gantt[key].alert(Drupal.t("You cannot edit this task."));
                  return false;
                }
                if (task.text.length > 255) {
                  Gantt[key].alert(Drupal.t("Description cannot be too long"));
                  return false;
                }
                return true;
              })

              Gantt[key].attachEvent("onLightboxDelete", function (id, task, is_new) {
                if (Gantt[key].isReadonly(id)) {
                  Gantt[key].alert(Drupal.t("You cannot delete this task."));
                  return false;
                }
                return true;
              })

              // Show progress %.
              if (ganttField.has_duration) {
                Gantt[key].templates.progress_text = function (start, end, task) {
                  return "<span style='text-align:left;'>" + Math.round(task.progress * 100) + "% </span>";
                };
              }

              // Add column +.
              if (!ganttField.add_task) {
                $.each(Gantt[key].config.columns, function (index, col) {
                  if (col.name === 'add') {
                    delete col.name;
                  }
                });
              }

              // Change mode view readonly.
              if (ganttField.edit_task === false) {
                Gantt[key].config.readonly = true;
              }

              // Switch dialog native drupal.
              if (ganttField.native_dialog) {
                native_dialog(Gantt[key], ganttField);
              }

              // Add custom column custom field.
              if (ganttField.custom_field) {
                let customColumnExist = Gantt[key].config.columns.findIndex((element) => element.name == ganttField.custom_field);
                if (customColumnExist < 0) {
                  Gantt[key].config.columns.splice(Gantt[key].config.columns.length - 1, 0, {
                    name: ganttField.custom_field,
                    label: ganttField.custom_field_name,
                    align: "center",
                    resize: true
                  });
                  Gantt[key].locale.labels['section_' + ganttField.custom_field] = ganttField.custom_field_name;
                }
                let customLightBoxExist = Gantt[key].config.lightbox.sections.findIndex((element) => element.name == ganttField.custom_field);
                if (customLightBoxExist < 0) {
                  Gantt[key].config.lightbox.sections.push(
                    {
                      name: ganttField.custom_field,
                      type: "number",
                      map_to: ganttField.custom_field,
                    }
                  );
                  Gantt[key].form_blocks["number"] = {
                    render: function (sns) {
                      return '<input class="form-control"  type="number" min="0">';
                    },
                    set_value: function (node, value, task) {
                      node.value = value;
                    },
                    get_value: function (node, task) {
                      return node.value || task.options;
                    },
                    focus: function (node) {
                    }
                  };
                }
              }

              // Add baseline on row.
              // Show 2nd chart for actual date or baseline.
              if (ganttField.has_planned && !ganttField.split_actual_planned && !ganttField.use_cdn) {
                Gantt[key].config.bar_height = 16;
                Gantt[key].config.row_height = 40;

                /* Adding baseline display. */
                if (Gantt[key].hasOwnProperty('addTaskLayer')) {
                  Gantt[key].addTaskLayer({
                    renderer: {
                      render: function draw_planned(task) {
                        /* Add class link when exist base line. */
                        Gantt[key].templates.link_class = function (link) {
                          return "exist_base_line_link";
                        };
                        /* Add base line. */
                        if (!isNaN(task.planned_start.getTime()) && !isNaN(task.planned_end.getTime())) {
                          let sizes = Gantt[key].getTaskPosition(task, task.planned_start, task.planned_end);
                          let el = document.createElement('div');
                          el.className = 'baseline';
                          el.style.left = sizes.left + 'px';
                          el.style.width = sizes.width + 'px';
                          el.style.top = sizes.top + Gantt[key].config.bar_height + 13 + 'px';
                          return el;
                        }
                        return false;
                      },
                      /* Define getRectangle in order to hook layer with the. */
                      getRectangle: function (task, view) {
                        if (task.planned_start && task.planned_end) {
                          return Gantt[key].getTaskPosition(task, task.planned_start, task.planned_end);
                        }
                        return null;
                      }
                    }
                  });

                  /* Calculator Overdue. */
                  Gantt[key].templates.rightside_text = function (start, end, task) {
                    if (task.planned_end) {
                      if (end.getTime() > task.planned_end.getTime()) {
                        let overdue = Math.ceil(Math.abs((end.getTime() - task.planned_end.getTime()) / (24 * 60 * 60 * 1000)));
                        let text = `<b>${Drupal.t('Overdue')}: ${overdue} ${Drupal.t('days')}</b>`;
                        return text;
                      }
                    }
                  };

                  Gantt[key].attachEvent("onTaskLoading", function (task) {
                    if (task.hasOwnProperty("planned_start") && task.hasOwnProperty("planned_end")) {
                      task.planned_start = Gantt[key].date.parseDate(new Date(task.planned_start), "xml_date");
                      task.planned_end = Gantt[key].date.parseDate(new Date(task.planned_end), "xml_date");
                    }
                    return true;
                  });
                }
              }

              // Add workTime and holiday.
              if (ganttField.work_time) {
                Gantt[key].config.work_time = true;

                /* Add holiday. */
                if (ganttField.holidays) {
                  let holidays = ganttField.holidays.map(dateString => {
                    let dateParts = dateString.split('-');
                    let year = parseInt(dateParts[0]);
                    let month = parseInt(dateParts[1]) - 1;
                    let day = parseInt(dateParts[2]);
                    Gantt[key].setWorkTime({
                      date: new Date(year, month, day),
                      hours: false
                    });
                    return new Date(year, month, day);
                  });
                }
                if (ganttField.work_day.length) {
                  Gantt[key].setWorkTime({day: 0, hours: true});
                  Gantt[key].setWorkTime({day: 6, hours: true});
                  $.each(ganttField.work_day, function (index, valueItem) {
                    Gantt[key].setWorkTime({day: valueItem, hours: false});
                  });
                }
                Gantt[key].templates.timeline_cell_class = function (task, date) {
                  if (!Gantt[key].isWorkTime({task: task, date: date})) {
                    return "weekend";
                  }
                };
              }

              // Set zoom config for charts.
              let zoomConfig = {
                minColumnWidth: 50,
                maxColumnWidth: 150,
                levels: [
                  {
                    name: "year",
                    scales: [
                      {unit: "year", format: "%Y", step: 1},
                    ]
                  },
                  {
                    name: "quarter",
                    scales: [
                      {unit: "year", format: "%Y", step: 1},
                      {
                        unit: "quarter", step: 1, format: function (date) {
                          let dateToStr = gantt.date.date_to_str("%M");
                          let endDate = gantt.date.add(gantt.date.add(date, 3, "month"), -1, "day");
                          return dateToStr(date) + " - " + dateToStr(endDate);
                        }
                      }
                    ]
                  },
                  {
                    name: "month",
                    scales: [
                      {unit: "year", format: "%Y", step: 1},
                      {unit: "month", format: "%M", step: 1}
                    ]
                  },
                  {
                    name: "week",
                    scales: [
                      {unit: "month", format: "%M %Y", step: 1},
                      {
                        unit: "week", step: 1, format: function (date) {
                          let dateToStr = gantt.date.date_to_str("%d/%m");
                          let endDate = gantt.date.add(gantt.date.add(date, 7, "day"), -1, "day");
                          let weekNum = gantt.date.date_to_str("%W")(date);
                          return "#" + weekNum + ", " + dateToStr(date) + " - " + dateToStr(endDate);
                        }
                      }
                    ]
                  },
                  {
                    name: "day",
                    scales: [
                      {unit: "month", format: "%M %Y", step: 1},
                      {unit: "day", format: "%d %M", step: 1}
                    ]
                  }
                ],
                useKey: "ctrlKey",
                trigger: "wheel",
                element: function () {
                  return Gantt[key].$root.querySelector(".gantt_task");
                }
              }

              Gantt[key].ext.zoom.init(zoomConfig);
              Gantt[key].ext.zoom.setLevel("day");
              Gantt[key].ext.zoom.attachEvent("onAfterZoom", function (level, config) {
                Gantt[key].$root.closest(".gantt-wrapper").querySelector("[onchange^='set_scale']").value = config.name;
              })

              // Add show column end Date.
              if (ganttField.show_end) {
                let indexCol = Gantt[key].config.columns.findIndex((item) => item.name == 'start_date');
                Gantt[key].config.columns.splice(indexCol + 1, 0, {
                  name: "end_date",
                  width: 80,
                  label: Drupal.t("End"),
                  align: "center",
                  resize: true,
                  template: function (task) {
                    return Gantt[key].templates.date_grid(task.end_date)
                  }
                });
              }

              // Owner.
              if (ganttField.owner) {
                owners(Gantt[key], ownerGantt[ganttField.id]);
              }

              // Set resource.
              if (ganttField.custom_resource.length) {
                ganttField.custom_resource.map(function (id_resource, index) {
                  resource(Gantt[key], resourceGantt[ganttField.id][id_resource], ganttField['setting_resource']['resource_label'][id_resource], id_resource, ganttField['setting_resource']['resource_column'], ganttField['setting_resource']['resource_lightbox']);
                })
              }

              // Only links.
              Gantt[key].attachEvent("onBeforeLinkAdd", function (id, link) {
                let taskSource = Gantt[key].getTask(link.source);
                let taskTarget = Gantt[key].getTask(link.target);
                if (Gantt[key].isReadonly(link.target)) {
                  Gantt[key].message({
                    type: "error",
                    text: Drupal.t("Can not link. <b>" + taskTarget.text + "</b> not permission."),
                    expire: 4000
                  });
                  return false;
                }
                if (taskSource.$source.includes(link.source + "-" + link.target) || taskTarget.$source.includes(link.target + "-" + link.source)) {
                  Gantt[key].message({
                    type: "error",
                    text: Drupal.t("Link <b>" + taskSource.text + "</b> to <b>" + taskTarget.text + "</b> already exist."),
                    expire: 4000
                  });
                  return false;
                }
                return true;
              });

              // Automatic scheduling.
              if (ganttField.auto_schedule && !ganttField.use_cdn) {
                linkLagEditor(Gantt[key]);

                Gantt[key].config.auto_scheduling = true;
                Gantt[key].attachEvent("onAfterTaskAutoSchedule", function (task, new_date, constraint, predecessor) {
                  if (task && predecessor) {
                    Gantt[key].message({
                      text: Drupal.t("<b>" + task.text + "</b> has been rescheduled to " + Gantt[key].templates.task_date(new_date) + " due to <b>" + predecessor.text + "</b> constraint"),
                      expire: 4000
                    });
                  }
                });
              }

              // Display task search text.
              Gantt[key].attachEvent("onBeforeTaskDisplay", function (id, task) {
                if (compare_input(Gantt[key], key, id)) {
                  return true;
                }
                return false;
              });

              // Format end date.
              Gantt[key].templates.task_end_date = function (date) {
                // Option Last of the day.
                if (ganttField.last_of_the_day) {
                  return Gantt[key].templates.task_date(new Date(date.valueOf() - 1));
                }
                return Gantt[key].templates.task_date(new Date(date.valueOf()));
              };
              let gridDateToStr = Gantt[key].date.date_to_str("%d-%m-%Y");
              Gantt[key].templates.grid_date_format = function (date, column) {
                if (column === "end_date") {
                  // Option Last of the day.
                  if (ganttField.last_of_the_day) {
                    return gridDateToStr(new Date(date.valueOf() - 1));
                  }
                  return gridDateToStr(new Date(date.valueOf()));
                }
                else {
                  return gridDateToStr(date);
                }
              }

              // Error ajax.
              Gantt[key].attachEvent("onAjaxError", function (request) {
                Gantt[key].message({
                  type: "error",
                  text: "Error " + request.status + "!",
                  expire: 4000
                })
                return true;
              });

              // Trigger gantt for mono chart.
              Gantt[key].createDataProcessor(configAjax);
              if ($('#' + ganttField.id).length) {
                Gantt[key].init(ganttField.id);
              }
              if ($('#' + ganttField.id).length && dataGantt[ganttField.id].data !== undefined) {
                Gantt[key].parse(dataGantt[ganttField.id]);
              }

              importMMP(Gantt[key], ganttField.id, ganttField.import);

              // add overlay chart.
              if (ganttField.chart_waste) {
                overlayControl[ganttField.id] = Gantt[key].ext.overlay;

                addChartJs(ganttField.id);
              }

              if (ganttField.split_actual_planned && typeof dataPlanned !== 'undefined' && dataPlanned[ganttField.id] !== undefined) {
                // Show 2nd chart for planed date.
                Gantt[key + 'plan'] = Gantt.getGanttInstance();
                Gantt[key + 'plan'].config = gantt.config;
                Gantt[key + 'plan'].createDataProcessor({
                  url: ganttField.ajax + '?isPlanned=1',
                  mode: "POST"
                });
                Gantt[key + 'plan'].init('planned-' + ganttField.id);
                Gantt[key + 'plan'].parse(dataPlanned[ganttField.id]);

                Gantt[key + 'plan'].plugins({
                  tooltip: true,
                });
              }
            }
          });
        }
      });
    },
  };
})(jQuery, Drupal, window.Cookies);
